# This file was auto-generated by Fern from our API Definition.

import time
import typing

from athena.types.message_out_dto import MessageOutDto

from .core import RequestOptions
from .message.client import AsyncMessageClient, MessageClient
from .types.model import Model
from .types.tools import Tools
from .types.status_enum import StatusEnum

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class MessagePollingClient(MessageClient):

    def submit_and_poll(
        self,
        *,
        content: str,
        model: typing.Optional[Model] = OMIT,
        tools: typing.Optional[typing.Sequence[Tools]] = OMIT,
        conversation_id: typing.Optional[str] = OMIT,
        conversation_name: typing.Optional[str] = OMIT,
        poll_interval: int = 2,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> MessageOutDto:
        """
        Parameters:
            - content: str.

            - model: typing.Optional[Model].

            - tools: typing.Optional[typing.Sequence[Tools]].

            - conversation_id: typing.Optional[str].

            - conversation_name: typing.Optional[str].

            - poll_interval: int. The polling interval in seconds.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from athena import Model, Tools
        from athena.client import Athena

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.message.submit(
            content="visit www.athenaintelligence.ai and summarize the website in one paragraph",
            model=Model.GPT_4_TURBO_PREVIEW,
            tools=[Tools.SEARCH, Tools.BROWSE],
        )
        """
        response = super().submit(
          content=content,
          model=model,
          tools=tools,
          conversation_id=conversation_id, 
          conversation_name=conversation_name,
          request_options=request_options)
        message = self.get(id=response.id)
        while message.status == StatusEnum.PENDING:
            time.sleep(poll_interval)
            message = self.get(id=message.id)
        return message


class AsyncMessagePollingClient(AsyncMessageClient):

    async def submit_and_poll(
        self,
        *,
        content: str,
        model: typing.Optional[Model] = OMIT,
        tools: typing.Optional[typing.Sequence[Tools]] = OMIT,
        conversation_id: typing.Optional[str] = OMIT,
        conversation_name: typing.Optional[str] = OMIT,
        poll_interval: int = 2,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> MessageOutDto: 
        """
        Parameters:
            - content: str.

            - model: typing.Optional[Model].

            - tools: typing.Optional[typing.Sequence[Tools]].

            - conversation_id: typing.Optional[str].

            - conversation_name: typing.Optional[str].

            - poll_interval: int. The polling interval in seconds.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from athena import Model, Tools
        from athena.client import AsyncAthena

        client = AsyncAthena(
            api_key="YOUR_API_KEY"
        )
        client.message.submit(
            content="visit www.athenaintelligence.ai and summarize the website in one paragraph",
            model=Model.GPT_4_TURBO_PREVIEW,
            tools=[Tools.SEARCH, Tools.BROWSE],
        )
        """
        response = await super().submit(
          content=content,
          model=model,
          tools=tools,
          conversation_id=conversation_id, 
          conversation_name=conversation_name,
          request_options=request_options)
        message = await self.get(id=response.id)
        while message.status == StatusEnum.PENDING:
            time.sleep(poll_interval)
            message = await self.get(id=message.id)
        return message



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class StatusEnum(str, enum.Enum):
    """
    An enumeration.
    """

    PENDING = "pending"
    RESPONDED = "responded"
    ERROR = "error"

    def visit(
        self,
        pending: typing.Callable[[], T_Result],
        responded: typing.Callable[[], T_Result],
        error: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is StatusEnum.PENDING:
            return pending()
        if self is StatusEnum.RESPONDED:
            return responded()
        if self is StatusEnum.ERROR:
            return error()



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import pydantic_v1
from ..core.remove_none_from_dict import remove_none_from_dict
from ..core.request_options import RequestOptions
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.http_validation_error import HttpValidationError
from ..types.message_out import MessageOut
from ..types.message_out_dto import MessageOutDto
from ..types.model import Model
from ..types.tools import Tools

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class MessageClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def submit(
        self,
        *,
        content: str,
        model: typing.Optional[Model] = OMIT,
        tools: typing.Optional[typing.Sequence[Tools]] = OMIT,
        conversation_id: typing.Optional[str] = OMIT,
        conversation_name: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> MessageOut:
        """
        Parameters:
            - content: str.

            - model: typing.Optional[Model].

            - tools: typing.Optional[typing.Sequence[Tools]].

            - conversation_id: typing.Optional[str].

            - conversation_name: typing.Optional[str].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from athena import Model, Tools
        from athena.client import Athena

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.message.submit(
            content="visit www.athenaintelligence.ai and summarize the website in one paragraph",
            model=Model.GPT_35_TURBO,
            tools=[Tools.SEARCH, Tools.BROWSE],
        )
        """
        _request: typing.Dict[str, typing.Any] = {"content": content}
        if model is not OMIT:
            _request["model"] = model
        if tools is not OMIT:
            _request["tools"] = tools
        if conversation_id is not OMIT:
            _request["conversation_id"] = conversation_id
        if conversation_name is not OMIT:
            _request["conversation_name"] = conversation_name
        _response = self._client_wrapper.httpx_client.request(
            method="POST",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/v0/message"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(MessageOut, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> MessageOutDto:
        """
        Parameters:
            - id: str.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from athena.client import Athena

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.message.get(
            id="id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/v0/message/{jsonable_encoder(id)}"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(MessageOutDto, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncMessageClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def submit(
        self,
        *,
        content: str,
        model: typing.Optional[Model] = OMIT,
        tools: typing.Optional[typing.Sequence[Tools]] = OMIT,
        conversation_id: typing.Optional[str] = OMIT,
        conversation_name: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> MessageOut:
        """
        Parameters:
            - content: str.

            - model: typing.Optional[Model].

            - tools: typing.Optional[typing.Sequence[Tools]].

            - conversation_id: typing.Optional[str].

            - conversation_name: typing.Optional[str].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from athena import Model, Tools
        from athena.client import AsyncAthena

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )
        await client.message.submit(
            content="visit www.athenaintelligence.ai and summarize the website in one paragraph",
            model=Model.GPT_35_TURBO,
            tools=[Tools.SEARCH, Tools.BROWSE],
        )
        """
        _request: typing.Dict[str, typing.Any] = {"content": content}
        if model is not OMIT:
            _request["model"] = model
        if tools is not OMIT:
            _request["tools"] = tools
        if conversation_id is not OMIT:
            _request["conversation_id"] = conversation_id
        if conversation_name is not OMIT:
            _request["conversation_name"] = conversation_name
        _response = await self._client_wrapper.httpx_client.request(
            method="POST",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/v0/message"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(MessageOut, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> MessageOutDto:
        """
        Parameters:
            - id: str.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from athena.client import AsyncAthena

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )
        await client.message.get(
            id="id",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", f"api/v0/message/{jsonable_encoder(id)}"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(MessageOutDto, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from .types import (
    ConvertPdfToSheetOut,
    Dataset,
    Document,
    ExcecuteToolFirstWorkflowOut,
    FileDataResponse,
    FilterModel,
    FilterOperator,
    FirecrawlScrapeUrlDataReponseDto,
    FirecrawlScrapeUrlMetadata,
    GetDatasetsResponse,
    GetSnippetOut,
    GetSnippetsResponse,
    HttpValidationError,
    LangchainDocumentsRequestOut,
    LlmModel,
    MapReduceChainOut,
    MessageOut,
    MessageOutDto,
    Model,
    PublishFormats,
    QueryModel,
    Report,
    ResearcherOut,
    SemanticQueryOut,
    Snippet,
    SqlResults,
    StatusEnum,
    StructuredParseResult,
    TimeDimensionModel,
    Tools,
    UploadDocumentsOut,
    UrlResult,
    ValidationError,
    ValidationErrorLocItem,
    WorkflowStatusOut,
)
from .errors import UnprocessableEntityError
from . import chain, dataset, message, query, report, search, snippet, tools, upload, workflow
from .environment import AthenaEnvironment
from .version import __version__

__all__ = [
    "AthenaEnvironment",
    "ConvertPdfToSheetOut",
    "Dataset",
    "Document",
    "ExcecuteToolFirstWorkflowOut",
    "FileDataResponse",
    "FilterModel",
    "FilterOperator",
    "FirecrawlScrapeUrlDataReponseDto",
    "FirecrawlScrapeUrlMetadata",
    "GetDatasetsResponse",
    "GetSnippetOut",
    "GetSnippetsResponse",
    "HttpValidationError",
    "LangchainDocumentsRequestOut",
    "LlmModel",
    "MapReduceChainOut",
    "MessageOut",
    "MessageOutDto",
    "Model",
    "PublishFormats",
    "QueryModel",
    "Report",
    "ResearcherOut",
    "SemanticQueryOut",
    "Snippet",
    "SqlResults",
    "StatusEnum",
    "StructuredParseResult",
    "TimeDimensionModel",
    "Tools",
    "UnprocessableEntityError",
    "UploadDocumentsOut",
    "UrlResult",
    "ValidationError",
    "ValidationErrorLocItem",
    "WorkflowStatusOut",
    "__version__",
    "chain",
    "dataset",
    "message",
    "query",
    "report",
    "search",
    "snippet",
    "tools",
    "upload",
    "workflow",
]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class LlmModel(str, enum.Enum):
    """
    An enumeration.
    """

    GPT_35_TURBO = "gpt-3.5-turbo"
    GPT_4_TURBO = "gpt-4-turbo"
    GPT_4_TURBO_PREVIEW = "gpt-4-turbo-preview"
    GPT_4_O_MINI = "gpt-4o-mini"
    GPT_4_O = "gpt-4o"
    GPT_4 = "gpt-4"
    MIXTRAL_SMALL_8_X_7_B_0211 = "mixtral-small-8x7b-0211"
    MISTRAL_LARGE_0224 = "mistral-large-0224"
    MIXTRAL_8_X_22_B_INSTRUCT = "mixtral-8x22b-instruct"
    LLAMA_V_38_B_INSTRUCT = "llama-v3-8b-instruct"
    LLAMA_V_370_B_INSTRUCT = "llama-v3-70b-instruct"
    CLAUDE_35_SONNET_20240620 = "claude-3-5-sonnet-20240620"
    CLAUDE_3_OPUS_20240229 = "claude-3-opus-20240229"
    CLAUDE_3_SONNET_20240229 = "claude-3-sonnet-20240229"
    CLAUDE_3_HAIKU_20240307 = "claude-3-haiku-20240307"
    GROQ_MIXTRAL_8_X_7_B_32768 = "groq-mixtral-8x7b-32768"
    GROQ_LLAMA_38_B_8192 = "groq-llama3-8b-8192"
    GROQ_LLAMA_370_B_8192 = "groq-llama3-70b-8192"
    GROQ_GEMMA_7_B_IT = "groq-gemma-7b-it"
    DATABRICKS_DBRX = "databricks-dbrx"
    GOOGLE_GEMINI_10_PRO_LATEST = "google-gemini-1.0-pro-latest"
    GOOGLE_GEMINI_15_PRO_LATEST = "google-gemini-1.5-pro-latest"
    GOOGLE_GEMINI_15_FLASH_001 = "google-gemini-1.5-flash-001"
    GOOGLE_GEMINI_15_PRO_001 = "google-gemini-1.5-pro-001"

    def visit(
        self,
        gpt_35_turbo: typing.Callable[[], T_Result],
        gpt_4_turbo: typing.Callable[[], T_Result],
        gpt_4_turbo_preview: typing.Callable[[], T_Result],
        gpt_4_o_mini: typing.Callable[[], T_Result],
        gpt_4_o: typing.Callable[[], T_Result],
        gpt_4: typing.Callable[[], T_Result],
        mixtral_small_8_x_7_b_0211: typing.Callable[[], T_Result],
        mistral_large_0224: typing.Callable[[], T_Result],
        mixtral_8_x_22_b_instruct: typing.Callable[[], T_Result],
        llama_v_38_b_instruct: typing.Callable[[], T_Result],
        llama_v_370_b_instruct: typing.Callable[[], T_Result],
        claude_35_sonnet_20240620: typing.Callable[[], T_Result],
        claude_3_opus_20240229: typing.Callable[[], T_Result],
        claude_3_sonnet_20240229: typing.Callable[[], T_Result],
        claude_3_haiku_20240307: typing.Callable[[], T_Result],
        groq_mixtral_8_x_7_b_32768: typing.Callable[[], T_Result],
        groq_llama_38_b_8192: typing.Callable[[], T_Result],
        groq_llama_370_b_8192: typing.Callable[[], T_Result],
        groq_gemma_7_b_it: typing.Callable[[], T_Result],
        databricks_dbrx: typing.Callable[[], T_Result],
        google_gemini_10_pro_latest: typing.Callable[[], T_Result],
        google_gemini_15_pro_latest: typing.Callable[[], T_Result],
        google_gemini_15_flash_001: typing.Callable[[], T_Result],
        google_gemini_15_pro_001: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is LlmModel.GPT_35_TURBO:
            return gpt_35_turbo()
        if self is LlmModel.GPT_4_TURBO:
            return gpt_4_turbo()
        if self is LlmModel.GPT_4_TURBO_PREVIEW:
            return gpt_4_turbo_preview()
        if self is LlmModel.GPT_4_O_MINI:
            return gpt_4_o_mini()
        if self is LlmModel.GPT_4_O:
            return gpt_4_o()
        if self is LlmModel.GPT_4:
            return gpt_4()
        if self is LlmModel.MIXTRAL_SMALL_8_X_7_B_0211:
            return mixtral_small_8_x_7_b_0211()
        if self is LlmModel.MISTRAL_LARGE_0224:
            return mistral_large_0224()
        if self is LlmModel.MIXTRAL_8_X_22_B_INSTRUCT:
            return mixtral_8_x_22_b_instruct()
        if self is LlmModel.LLAMA_V_38_B_INSTRUCT:
            return llama_v_38_b_instruct()
        if self is LlmModel.LLAMA_V_370_B_INSTRUCT:
            return llama_v_370_b_instruct()
        if self is LlmModel.CLAUDE_35_SONNET_20240620:
            return claude_35_sonnet_20240620()
        if self is LlmModel.CLAUDE_3_OPUS_20240229:
            return claude_3_opus_20240229()
        if self is LlmModel.CLAUDE_3_SONNET_20240229:
            return claude_3_sonnet_20240229()
        if self is LlmModel.CLAUDE_3_HAIKU_20240307:
            return claude_3_haiku_20240307()
        if self is LlmModel.GROQ_MIXTRAL_8_X_7_B_32768:
            return groq_mixtral_8_x_7_b_32768()
        if self is LlmModel.GROQ_LLAMA_38_B_8192:
            return groq_llama_38_b_8192()
        if self is LlmModel.GROQ_LLAMA_370_B_8192:
            return groq_llama_370_b_8192()
        if self is LlmModel.GROQ_GEMMA_7_B_IT:
            return groq_gemma_7_b_it()
        if self is LlmModel.DATABRICKS_DBRX:
            return databricks_dbrx()
        if self is LlmModel.GOOGLE_GEMINI_10_PRO_LATEST:
            return google_gemini_10_pro_latest()
        if self is LlmModel.GOOGLE_GEMINI_15_PRO_LATEST:
            return google_gemini_15_pro_latest()
        if self is LlmModel.GOOGLE_GEMINI_15_FLASH_001:
            return google_gemini_15_flash_001()
        if self is LlmModel.GOOGLE_GEMINI_15_PRO_001:
            return google_gemini_15_pro_001()



# This file was auto-generated by Fern from our API Definition.

from .convert_pdf_to_sheet_out import ConvertPdfToSheetOut
from .dataset import Dataset
from .document import Document
from .excecute_tool_first_workflow_out import ExcecuteToolFirstWorkflowOut
from .file_data_response import FileDataResponse
from .filter_model import FilterModel
from .filter_operator import FilterOperator
from .firecrawl_scrape_url_data_reponse_dto import FirecrawlScrapeUrlDataReponseDto
from .firecrawl_scrape_url_metadata import FirecrawlScrapeUrlMetadata
from .get_datasets_response import GetDatasetsResponse
from .get_snippet_out import GetSnippetOut
from .get_snippets_response import GetSnippetsResponse
from .http_validation_error import HttpValidationError
from .langchain_documents_request_out import LangchainDocumentsRequestOut
from .llm_model import LlmModel
from .map_reduce_chain_out import MapReduceChainOut
from .message_out import MessageOut
from .message_out_dto import MessageOutDto
from .model import Model
from .publish_formats import PublishFormats
from .query_model import QueryModel
from .report import Report
from .researcher_out import ResearcherOut
from .semantic_query_out import SemanticQueryOut
from .snippet import Snippet
from .sql_results import SqlResults
from .status_enum import StatusEnum
from .structured_parse_result import StructuredParseResult
from .time_dimension_model import TimeDimensionModel
from .tools import Tools
from .upload_documents_out import UploadDocumentsOut
from .url_result import UrlResult
from .validation_error import ValidationError
from .validation_error_loc_item import ValidationErrorLocItem
from .workflow_status_out import WorkflowStatusOut

__all__ = [
    "ConvertPdfToSheetOut",
    "Dataset",
    "Document",
    "ExcecuteToolFirstWorkflowOut",
    "FileDataResponse",
    "FilterModel",
    "FilterOperator",
    "FirecrawlScrapeUrlDataReponseDto",
    "FirecrawlScrapeUrlMetadata",
    "GetDatasetsResponse",
    "GetSnippetOut",
    "GetSnippetsResponse",
    "HttpValidationError",
    "LangchainDocumentsRequestOut",
    "LlmModel",
    "MapReduceChainOut",
    "MessageOut",
    "MessageOutDto",
    "Model",
    "PublishFormats",
    "QueryModel",
    "Report",
    "ResearcherOut",
    "SemanticQueryOut",
    "Snippet",
    "SqlResults",
    "StatusEnum",
    "StructuredParseResult",
    "TimeDimensionModel",
    "Tools",
    "UploadDocumentsOut",
    "UrlResult",
    "ValidationError",
    "ValidationErrorLocItem",
    "WorkflowStatusOut",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import httpx

from .base_client import BaseAthena, AsyncBaseAthena
from .environment import AthenaEnvironment
from .polling_message_client import \
        MessagePollingClient, AsyncMessagePollingClient


class Athena(BaseAthena):
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propogate to these functions.

    Parameters:
        - base_url: typing.Optional[str]. The base url to use for requests from the client.

        - environment: AthenaEnvironment. The environment to use for requests from the client. from .environment import AthenaEnvironment

                                          Defaults to AthenaEnvironment.DEFAULT

        - api_key: str.

        - timeout: typing.Optional[float]. The timeout to be used, in seconds, for requests by default the timeout is 60 seconds.

        - httpx_client: typing.Optional[httpx.Client]. The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.
    ---
    from athena.client import Athena

    client = Athena(
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: AthenaEnvironment = AthenaEnvironment.DEFAULT,
        api_key: str,
        timeout: typing.Optional[float] = 60,
        httpx_client: typing.Optional[httpx.Client] = None
    ):
        super().__init__(
            base_url=base_url,
            environment=environment,
            api_key=api_key,
            timeout=timeout,
            httpx_client=httpx_client,
        )
        self.message = MessagePollingClient(
            client_wrapper=self._client_wrapper)


class AsyncAthena(AsyncBaseAthena):
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propogate to these functions.

    Parameters:
        - base_url: typing.Optional[str]. The base url to use for requests from the client.

        - environment: AthenaEnvironment. The environment to use for requests from the client. from .environment import AthenaEnvironment

                                          Defaults to AthenaEnvironment.DEFAULT

        - api_key: str.

        - timeout: typing.Optional[float]. The timeout to be used, in seconds, for requests by default the timeout is 60 seconds.

        - httpx_client: typing.Optional[httpx.AsyncClient]. The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.
    ---
    from athena.client import AsyncAthena

    client = AsyncAthena(
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: AthenaEnvironment = AthenaEnvironment.DEFAULT,
        api_key: str,
        timeout: typing.Optional[float] = 60,
        httpx_client: typing.Optional[httpx.AsyncClient] = None
    ):
        super().__init__(
            base_url=base_url,
            environment=environment,
            api_key=api_key,
            timeout=timeout,
            httpx_client=httpx_client,
        )
        self.message = AsyncMessagePollingClient(
            client_wrapper=self._client_wrapper)




# This file was auto-generated by Fern from our API Definition.

import typing

import httpx

from .chain.client import AsyncChainClient, ChainClient
from .core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .dataset.client import AsyncDatasetClient, DatasetClient
from .environment import AthenaEnvironment
from .message.client import AsyncMessageClient, MessageClient
from .query.client import AsyncQueryClient, QueryClient
from .report.client import AsyncReportClient, ReportClient
from .search.client import AsyncSearchClient, SearchClient
from .snippet.client import AsyncSnippetClient, SnippetClient
from .tools.client import AsyncToolsClient, ToolsClient
from .upload.client import AsyncUploadClient, UploadClient
from .workflow.client import AsyncWorkflowClient, WorkflowClient


class BaseAthena:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propogate to these functions.

    Parameters:
        - base_url: typing.Optional[str]. The base url to use for requests from the client.

        - environment: AthenaEnvironment. The environment to use for requests from the client. from .environment import AthenaEnvironment

                                          Defaults to AthenaEnvironment.DEFAULT

        - api_key: str.

        - timeout: typing.Optional[float]. The timeout to be used, in seconds, for requests by default the timeout is 60 seconds, unless a custom httpx client is used, in which case a default is not set.

        - follow_redirects: typing.Optional[bool]. Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

        - httpx_client: typing.Optional[httpx.Client]. The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.
    ---
    from athena.client import Athena

    client = Athena(
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: AthenaEnvironment = AthenaEnvironment.DEFAULT,
        api_key: str,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None
    ):
        _defaulted_timeout = timeout if timeout is not None else 60 if httpx_client is None else None
        self._client_wrapper = SyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            api_key=api_key,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.Client(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self.message = MessageClient(client_wrapper=self._client_wrapper)
        self.dataset = DatasetClient(client_wrapper=self._client_wrapper)
        self.snippet = SnippetClient(client_wrapper=self._client_wrapper)
        self.report = ReportClient(client_wrapper=self._client_wrapper)
        self.query = QueryClient(client_wrapper=self._client_wrapper)
        self.search = SearchClient(client_wrapper=self._client_wrapper)
        self.chain = ChainClient(client_wrapper=self._client_wrapper)
        self.tools = ToolsClient(client_wrapper=self._client_wrapper)
        self.upload = UploadClient(client_wrapper=self._client_wrapper)
        self.workflow = WorkflowClient(client_wrapper=self._client_wrapper)


class AsyncBaseAthena:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propogate to these functions.

    Parameters:
        - base_url: typing.Optional[str]. The base url to use for requests from the client.

        - environment: AthenaEnvironment. The environment to use for requests from the client. from .environment import AthenaEnvironment

                                          Defaults to AthenaEnvironment.DEFAULT

        - api_key: str.

        - timeout: typing.Optional[float]. The timeout to be used, in seconds, for requests by default the timeout is 60 seconds, unless a custom httpx client is used, in which case a default is not set.

        - follow_redirects: typing.Optional[bool]. Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

        - httpx_client: typing.Optional[httpx.AsyncClient]. The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.
    ---
    from athena.client import AsyncAthena

    client = AsyncAthena(
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: AthenaEnvironment = AthenaEnvironment.DEFAULT,
        api_key: str,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None
    ):
        _defaulted_timeout = timeout if timeout is not None else 60 if httpx_client is None else None
        self._client_wrapper = AsyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            api_key=api_key,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self.message = AsyncMessageClient(client_wrapper=self._client_wrapper)
        self.dataset = AsyncDatasetClient(client_wrapper=self._client_wrapper)
        self.snippet = AsyncSnippetClient(client_wrapper=self._client_wrapper)
        self.report = AsyncReportClient(client_wrapper=self._client_wrapper)
        self.query = AsyncQueryClient(client_wrapper=self._client_wrapper)
        self.search = AsyncSearchClient(client_wrapper=self._client_wrapper)
        self.chain = AsyncChainClient(client_wrapper=self._client_wrapper)
        self.tools = AsyncToolsClient(client_wrapper=self._client_wrapper)
        self.upload = AsyncUploadClient(client_wrapper=self._client_wrapper)
        self.workflow = AsyncWorkflowClient(client_wrapper=self._client_wrapper)


def _get_base_url(*, base_url: typing.Optional[str] = None, environment: AthenaEnvironment) -> str:
    if base_url is not None:
        return base_url
    elif environment is not None:
        return environment.value
    else:
        raise Exception("Please pass in either base_url or environment to construct the client")



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import pydantic_v1
from ..core.remove_none_from_dict import remove_none_from_dict
from ..core.request_options import RequestOptions
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.convert_pdf_to_sheet_out import ConvertPdfToSheetOut
from ..types.excecute_tool_first_workflow_out import ExcecuteToolFirstWorkflowOut
from ..types.firecrawl_scrape_url_data_reponse_dto import FirecrawlScrapeUrlDataReponseDto
from ..types.http_validation_error import HttpValidationError
from ..types.langchain_documents_request_out import LangchainDocumentsRequestOut
from ..types.llm_model import LlmModel
from ..types.publish_formats import PublishFormats
from ..types.query_model import QueryModel
from ..types.researcher_out import ResearcherOut
from ..types.semantic_query_out import SemanticQueryOut

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ToolsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def scrape_url(
        self,
        *,
        url: str,
        params: typing.Optional[typing.Dict[str, typing.Any]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FirecrawlScrapeUrlDataReponseDto:
        """
        Parameters:
            - url: str.

            - params: typing.Optional[typing.Dict[str, typing.Any]].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from athena.client import Athena

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.tools.scrape_url(
            url="https://athenaintelligence.ai",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"url": url}
        if params is not OMIT:
            _request["params"] = params
        _response = self._client_wrapper.httpx_client.request(
            method="POST",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/v0/tools/firecrawl/scrape-url"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(FirecrawlScrapeUrlDataReponseDto, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def langchain_documents(
        self,
        *,
        document_id: str,
        pagination_limit: typing.Optional[int] = OMIT,
        pagination_offset: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> LangchainDocumentsRequestOut:
        """
        Parameters:
            - document_id: str.

            - pagination_limit: typing.Optional[int].

            - pagination_offset: typing.Optional[int].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from athena.client import Athena

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.tools.langchain_documents(
            document_id="doc_9249292-d118-42d3-95b4-00eccfe0754f",
            pagination_limit=250,
            pagination_offset=0,
        )
        """
        _request: typing.Dict[str, typing.Any] = {"document_id": document_id}
        if pagination_limit is not OMIT:
            _request["pagination_limit"] = pagination_limit
        if pagination_offset is not OMIT:
            _request["pagination_offset"] = pagination_offset
        _response = self._client_wrapper.httpx_client.request(
            method="POST",
            url=urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", "api/v0/tools/file/langchain-documents"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(LangchainDocumentsRequestOut, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def tool_first_workflow(
        self,
        *,
        model: typing.Optional[LlmModel] = OMIT,
        tool_name: str,
        content: str,
        tool_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExcecuteToolFirstWorkflowOut:
        """
        Parameters:
            - model: typing.Optional[LlmModel].

            - tool_name: str.

            - content: str.

            - tool_kwargs: typing.Optional[typing.Dict[str, typing.Any]].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from athena import LlmModel
        from athena.client import Athena

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.tools.tool_first_workflow(
            model=LlmModel.MISTRAL_LARGE_0224,
            tool_name="tavily_search",
            content="summarize the website in one paragraph",
            tool_kwargs={"query": "website: www.athenaintelligence.ai"},
        )
        """
        _request: typing.Dict[str, typing.Any] = {"tool_name": tool_name, "content": content}
        if model is not OMIT:
            _request["model"] = model
        if tool_kwargs is not OMIT:
            _request["tool_kwargs"] = tool_kwargs
        _response = self._client_wrapper.httpx_client.request(
            method="POST",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/v0/tools/first-agent"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ExcecuteToolFirstWorkflowOut, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def researcher(
        self,
        *,
        query: str,
        max_sections: int,
        guidelines: typing.Sequence[str],
        publish_formats: typing.Optional[PublishFormats] = OMIT,
        source: typing.Optional[str] = OMIT,
        athena_document_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResearcherOut:
        """
        Parameters:
            - query: str.

            - max_sections: int.

            - guidelines: typing.Sequence[str].

            - publish_formats: typing.Optional[PublishFormats].

            - source: typing.Optional[str].

            - athena_document_ids: typing.Optional[typing.Sequence[str]].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from athena import PublishFormats
        from athena.client import Athena

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.tools.researcher(
            query="Write a report about the company Athena Intelligence.",
            max_sections=10,
            guidelines=[
                "The report MUST be written in APA format",
                "Each sub section MUST include supporting sources using hyperlinks. If none exist, erase the sub section or rewrite it to be a part of the previous section",
                "The report MUST be written in english",
            ],
            publish_formats=PublishFormats(
                markdown=False,
                pdf=True,
                docx=False,
            ),
            source="web",
            athena_document_ids=["doc_1", "doc_2"],
        )
        """
        _request: typing.Dict[str, typing.Any] = {
            "query": query,
            "max_sections": max_sections,
            "guidelines": guidelines,
        }
        if publish_formats is not OMIT:
            _request["publish_formats"] = publish_formats
        if source is not OMIT:
            _request["source"] = source
        if athena_document_ids is not OMIT:
            _request["athena_document_ids"] = athena_document_ids
        _response = self._client_wrapper.httpx_client.request(
            method="POST",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/v0/tools/researcher"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ResearcherOut, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def convert_pdf_to_sheet(
        self, *, document_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> ConvertPdfToSheetOut:
        """
        Parameters:
            - document_id: str.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from athena.client import Athena

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.tools.convert_pdf_to_sheet(
            document_id="doc_9249292-d118-42d3-95b4-00eccfe0754f",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            method="POST",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/v0/tools/convert-pdf-to-sheet"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder({"document_id": document_id})
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder({"document_id": document_id}),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ConvertPdfToSheetOut, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def semantic_query(
        self,
        *,
        query: QueryModel,
        table_name: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SemanticQueryOut:
        """
        Parameters:
            - query: QueryModel.

            - table_name: typing.Optional[str].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from athena import FilterModel, FilterOperator, QueryModel, TimeDimensionModel
        from athena.client import Athena

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.tools.semantic_query(
            query=QueryModel(
                measures=["count"],
                time_dimensions=[
                    TimeDimensionModel(
                        dimension="dimension",
                        granularity="granularity",
                        date_range=["dateRange"],
                    )
                ],
                dimensions=["manufacturer"],
                filters=[
                    FilterModel(
                        dimension="manufacturer",
                        operator=FilterOperator.EQUALS,
                        values=["values"],
                    )
                ],
            ),
        )
        """
        _request: typing.Dict[str, typing.Any] = {"query": query}
        if table_name is not OMIT:
            _request["table_name"] = table_name
        _response = self._client_wrapper.httpx_client.request(
            method="POST",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/v0/tools/strict-semantic-query"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(SemanticQueryOut, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncToolsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def scrape_url(
        self,
        *,
        url: str,
        params: typing.Optional[typing.Dict[str, typing.Any]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FirecrawlScrapeUrlDataReponseDto:
        """
        Parameters:
            - url: str.

            - params: typing.Optional[typing.Dict[str, typing.Any]].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from athena.client import AsyncAthena

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )
        await client.tools.scrape_url(
            url="https://athenaintelligence.ai",
        )
        """
        _request: typing.Dict[str, typing.Any] = {"url": url}
        if params is not OMIT:
            _request["params"] = params
        _response = await self._client_wrapper.httpx_client.request(
            method="POST",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/v0/tools/firecrawl/scrape-url"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(FirecrawlScrapeUrlDataReponseDto, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def langchain_documents(
        self,
        *,
        document_id: str,
        pagination_limit: typing.Optional[int] = OMIT,
        pagination_offset: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> LangchainDocumentsRequestOut:
        """
        Parameters:
            - document_id: str.

            - pagination_limit: typing.Optional[int].

            - pagination_offset: typing.Optional[int].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from athena.client import AsyncAthena

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )
        await client.tools.langchain_documents(
            document_id="doc_9249292-d118-42d3-95b4-00eccfe0754f",
            pagination_limit=250,
            pagination_offset=0,
        )
        """
        _request: typing.Dict[str, typing.Any] = {"document_id": document_id}
        if pagination_limit is not OMIT:
            _request["pagination_limit"] = pagination_limit
        if pagination_offset is not OMIT:
            _request["pagination_offset"] = pagination_offset
        _response = await self._client_wrapper.httpx_client.request(
            method="POST",
            url=urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/", "api/v0/tools/file/langchain-documents"
            ),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(LangchainDocumentsRequestOut, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def tool_first_workflow(
        self,
        *,
        model: typing.Optional[LlmModel] = OMIT,
        tool_name: str,
        content: str,
        tool_kwargs: typing.Optional[typing.Dict[str, typing.Any]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExcecuteToolFirstWorkflowOut:
        """
        Parameters:
            - model: typing.Optional[LlmModel].

            - tool_name: str.

            - content: str.

            - tool_kwargs: typing.Optional[typing.Dict[str, typing.Any]].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from athena import LlmModel
        from athena.client import AsyncAthena

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )
        await client.tools.tool_first_workflow(
            model=LlmModel.MISTRAL_LARGE_0224,
            tool_name="tavily_search",
            content="summarize the website in one paragraph",
            tool_kwargs={"query": "website: www.athenaintelligence.ai"},
        )
        """
        _request: typing.Dict[str, typing.Any] = {"tool_name": tool_name, "content": content}
        if model is not OMIT:
            _request["model"] = model
        if tool_kwargs is not OMIT:
            _request["tool_kwargs"] = tool_kwargs
        _response = await self._client_wrapper.httpx_client.request(
            method="POST",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/v0/tools/first-agent"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ExcecuteToolFirstWorkflowOut, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def researcher(
        self,
        *,
        query: str,
        max_sections: int,
        guidelines: typing.Sequence[str],
        publish_formats: typing.Optional[PublishFormats] = OMIT,
        source: typing.Optional[str] = OMIT,
        athena_document_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResearcherOut:
        """
        Parameters:
            - query: str.

            - max_sections: int.

            - guidelines: typing.Sequence[str].

            - publish_formats: typing.Optional[PublishFormats].

            - source: typing.Optional[str].

            - athena_document_ids: typing.Optional[typing.Sequence[str]].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from athena import PublishFormats
        from athena.client import AsyncAthena

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )
        await client.tools.researcher(
            query="Write a report about the company Athena Intelligence.",
            max_sections=10,
            guidelines=[
                "The report MUST be written in APA format",
                "Each sub section MUST include supporting sources using hyperlinks. If none exist, erase the sub section or rewrite it to be a part of the previous section",
                "The report MUST be written in english",
            ],
            publish_formats=PublishFormats(
                markdown=False,
                pdf=True,
                docx=False,
            ),
            source="web",
            athena_document_ids=["doc_1", "doc_2"],
        )
        """
        _request: typing.Dict[str, typing.Any] = {
            "query": query,
            "max_sections": max_sections,
            "guidelines": guidelines,
        }
        if publish_formats is not OMIT:
            _request["publish_formats"] = publish_formats
        if source is not OMIT:
            _request["source"] = source
        if athena_document_ids is not OMIT:
            _request["athena_document_ids"] = athena_document_ids
        _response = await self._client_wrapper.httpx_client.request(
            method="POST",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/v0/tools/researcher"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ResearcherOut, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def convert_pdf_to_sheet(
        self, *, document_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> ConvertPdfToSheetOut:
        """
        Parameters:
            - document_id: str.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from athena.client import AsyncAthena

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )
        await client.tools.convert_pdf_to_sheet(
            document_id="doc_9249292-d118-42d3-95b4-00eccfe0754f",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            method="POST",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/v0/tools/convert-pdf-to-sheet"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder({"document_id": document_id})
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder({"document_id": document_id}),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(ConvertPdfToSheetOut, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def semantic_query(
        self,
        *,
        query: QueryModel,
        table_name: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SemanticQueryOut:
        """
        Parameters:
            - query: QueryModel.

            - table_name: typing.Optional[str].

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from athena import FilterModel, FilterOperator, QueryModel, TimeDimensionModel
        from athena.client import AsyncAthena

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )
        await client.tools.semantic_query(
            query=QueryModel(
                measures=["count"],
                time_dimensions=[
                    TimeDimensionModel(
                        dimension="dimension",
                        granularity="granularity",
                        date_range=["dateRange"],
                    )
                ],
                dimensions=["manufacturer"],
                filters=[
                    FilterModel(
                        dimension="manufacturer",
                        operator=FilterOperator.EQUALS,
                        values=["values"],
                    )
                ],
            ),
        )
        """
        _request: typing.Dict[str, typing.Any] = {"query": query}
        if table_name is not OMIT:
            _request["table_name"] = table_name
        _response = await self._client_wrapper.httpx_client.request(
            method="POST",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/v0/tools/strict-semantic-query"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(_request)
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(_request),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(SemanticQueryOut, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import pydantic_v1
from ..core.remove_none_from_dict import remove_none_from_dict
from ..core.request_options import RequestOptions
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.document import Document
from ..types.http_validation_error import HttpValidationError
from ..types.llm_model import LlmModel
from ..types.map_reduce_chain_out import MapReduceChainOut
from ..types.structured_parse_result import StructuredParseResult

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ChainClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def structured_parse(
        self,
        *,
        text_input: str,
        custom_type_dict: typing.Dict[str, typing.Any],
        model: LlmModel,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> StructuredParseResult:
        """
        Parameters:
            - text_input: str. The text input to be parsed.

            - custom_type_dict: typing.Dict[str, typing.Any]. A dictionary of field names and their default values.

            - model: LlmModel.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from athena import LlmModel
        from athena.client import Athena

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.chain.structured_parse(
            text_input='Athena is an AI-native analytics platform and artificial employee built to accelerate analytics workflows \n                by offering enterprise teams co-pilot and auto-pilot modes. Athena learns your workflow as a co-pilot, \n                allowing you to hand over controls to her for autonomous execution with confidence." \n                \n                Give me all of the modes Athena provides.',
            custom_type_dict={"modes": {}},
            model=LlmModel.GPT_4_TURBO,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            method="POST",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/v0/structured-parse"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder({"text_input": text_input, "custom_type_dict": custom_type_dict, "model": model})
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder({"text_input": text_input, "custom_type_dict": custom_type_dict, "model": model}),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(StructuredParseResult, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def map_reduce_chain(
        self,
        *,
        documents: typing.Sequence[Document],
        model: LlmModel,
        operator_prompt: str,
        reducer_prompt: str,
        input: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> MapReduceChainOut:
        """
        Parameters:
            - documents: typing.Sequence[Document].

            - model: LlmModel.

            - operator_prompt: str.

            - reducer_prompt: str.

            - input: str.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from athena import Document, LlmModel
        from athena.client import Athena

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.chain.map_reduce_chain(
            documents=[
                Document(
                    page_content="Athena is an AI-native analytics platform and artificial employee built to accelerate analytics workflows by offering enterprise teams co-pilot and auto-pilot modes. Athena learns your workflow as a co-pilot, allowing you to hand over controls to her for autonomous execution with confidence.",
                )
            ],
            model=LlmModel.MISTRAL_LARGE_0224,
            operator_prompt="summarize the content",
            reducer_prompt="Combine these summaries",
            input="return a summary in a single sentence",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            method="POST",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/v0/tools/map-reduce"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(
                {
                    "documents": documents,
                    "model": model,
                    "operator_prompt": operator_prompt,
                    "reducer_prompt": reducer_prompt,
                    "input": input,
                }
            )
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(
                    {
                        "documents": documents,
                        "model": model,
                        "operator_prompt": operator_prompt,
                        "reducer_prompt": reducer_prompt,
                        "input": input,
                    }
                ),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(MapReduceChainOut, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncChainClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def structured_parse(
        self,
        *,
        text_input: str,
        custom_type_dict: typing.Dict[str, typing.Any],
        model: LlmModel,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> StructuredParseResult:
        """
        Parameters:
            - text_input: str. The text input to be parsed.

            - custom_type_dict: typing.Dict[str, typing.Any]. A dictionary of field names and their default values.

            - model: LlmModel.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from athena import LlmModel
        from athena.client import AsyncAthena

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )
        await client.chain.structured_parse(
            text_input='Athena is an AI-native analytics platform and artificial employee built to accelerate analytics workflows \n                by offering enterprise teams co-pilot and auto-pilot modes. Athena learns your workflow as a co-pilot, \n                allowing you to hand over controls to her for autonomous execution with confidence." \n                \n                Give me all of the modes Athena provides.',
            custom_type_dict={"modes": {}},
            model=LlmModel.GPT_4_TURBO,
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            method="POST",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/v0/structured-parse"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder({"text_input": text_input, "custom_type_dict": custom_type_dict, "model": model})
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder({"text_input": text_input, "custom_type_dict": custom_type_dict, "model": model}),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(StructuredParseResult, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def map_reduce_chain(
        self,
        *,
        documents: typing.Sequence[Document],
        model: LlmModel,
        operator_prompt: str,
        reducer_prompt: str,
        input: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> MapReduceChainOut:
        """
        Parameters:
            - documents: typing.Sequence[Document].

            - model: LlmModel.

            - operator_prompt: str.

            - reducer_prompt: str.

            - input: str.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from athena import Document, LlmModel
        from athena.client import AsyncAthena

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )
        await client.chain.map_reduce_chain(
            documents=[
                Document(
                    page_content="Athena is an AI-native analytics platform and artificial employee built to accelerate analytics workflows by offering enterprise teams co-pilot and auto-pilot modes. Athena learns your workflow as a co-pilot, allowing you to hand over controls to her for autonomous execution with confidence.",
                )
            ],
            model=LlmModel.MISTRAL_LARGE_0224,
            operator_prompt="summarize the content",
            reducer_prompt="Combine these summaries",
            input="return a summary in a single sentence",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            method="POST",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/v0/tools/map-reduce"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder(
                {
                    "documents": documents,
                    "model": model,
                    "operator_prompt": operator_prompt,
                    "reducer_prompt": reducer_prompt,
                    "input": input,
                }
            )
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder(
                    {
                        "documents": documents,
                        "model": model,
                        "operator_prompt": operator_prompt,
                        "reducer_prompt": reducer_prompt,
                        "input": input,
                    }
                ),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(MapReduceChainOut, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class FilterOperator(str, enum.Enum):
    """
    An enumeration.
    """

    EQUALS = "equals"
    NOT_EQUALS = "notEquals"
    CONTAINS = "contains"
    NOT_CONTAINS = "notContains"
    STARTS_WITH = "startsWith"
    ENDS_WITH = "endsWith"
    GT = "gt"
    GTE = "gte"
    LT = "lt"
    LTE = "lte"
    SET = "set"
    NOT_SET = "notSet"
    IN = "in"
    NOT_IN = "notIn"

    def visit(
        self,
        equals: typing.Callable[[], T_Result],
        not_equals: typing.Callable[[], T_Result],
        contains: typing.Callable[[], T_Result],
        not_contains: typing.Callable[[], T_Result],
        starts_with: typing.Callable[[], T_Result],
        ends_with: typing.Callable[[], T_Result],
        gt: typing.Callable[[], T_Result],
        gte: typing.Callable[[], T_Result],
        lt: typing.Callable[[], T_Result],
        lte: typing.Callable[[], T_Result],
        set_: typing.Callable[[], T_Result],
        not_set: typing.Callable[[], T_Result],
        in_: typing.Callable[[], T_Result],
        not_in: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is FilterOperator.EQUALS:
            return equals()
        if self is FilterOperator.NOT_EQUALS:
            return not_equals()
        if self is FilterOperator.CONTAINS:
            return contains()
        if self is FilterOperator.NOT_CONTAINS:
            return not_contains()
        if self is FilterOperator.STARTS_WITH:
            return starts_with()
        if self is FilterOperator.ENDS_WITH:
            return ends_with()
        if self is FilterOperator.GT:
            return gt()
        if self is FilterOperator.GTE:
            return gte()
        if self is FilterOperator.LT:
            return lt()
        if self is FilterOperator.LTE:
            return lte()
        if self is FilterOperator.SET:
            return set_()
        if self is FilterOperator.NOT_SET:
            return not_set()
        if self is FilterOperator.IN:
            return in_()
        if self is FilterOperator.NOT_IN:
            return not_in()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class Tools(str, enum.Enum):
    """
    An enumeration.
    """

    SEARCH = "search"
    BROWSE = "browse"
    WIKIPEDIA = "wikipedia"
    DATABASE_METADATA = "database_metadata"
    QUERY = "query"
    CHART = "chart"
    ENRICH_PERSON = "enrich_person"
    ENRICH_COMPANY = "enrich_company"
    BROWSERBASE = "browserbase"

    def visit(
        self,
        search: typing.Callable[[], T_Result],
        browse: typing.Callable[[], T_Result],
        wikipedia: typing.Callable[[], T_Result],
        database_metadata: typing.Callable[[], T_Result],
        query: typing.Callable[[], T_Result],
        chart: typing.Callable[[], T_Result],
        enrich_person: typing.Callable[[], T_Result],
        enrich_company: typing.Callable[[], T_Result],
        browserbase: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is Tools.SEARCH:
            return search()
        if self is Tools.BROWSE:
            return browse()
        if self is Tools.WIKIPEDIA:
            return wikipedia()
        if self is Tools.DATABASE_METADATA:
            return database_metadata()
        if self is Tools.QUERY:
            return query()
        if self is Tools.CHART:
            return chart()
        if self is Tools.ENRICH_PERSON:
            return enrich_person()
        if self is Tools.ENRICH_COMPANY:
            return enrich_company()
        if self is Tools.BROWSERBASE:
            return browserbase()



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import pydantic_v1
from ..core.remove_none_from_dict import remove_none_from_dict
from ..core.request_options import RequestOptions
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.get_snippet_out import GetSnippetOut
from ..types.get_snippets_response import GetSnippetsResponse
from ..types.http_validation_error import HttpValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class SnippetClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self,
        *,
        page: typing.Optional[int] = None,
        page_size: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetSnippetsResponse:
        """
        Parameters:
            - page: typing.Optional[int]. Page number starting from 1

            - page_size: typing.Optional[int]. Number of items per page

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from athena.client import Athena

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.snippet.get()
        """
        _response = self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/v0/snippets"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "page": page,
                        "page_size": page_size,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(GetSnippetsResponse, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_by_id(self, *, id: str, request_options: typing.Optional[RequestOptions] = None) -> GetSnippetOut:
        """
        Parameters:
            - id: str.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from athena.client import Athena

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.snippet.get_by_id(
            id="snippet_02329f1b-f6ad-4e93-be84-355c33202024b",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            method="POST",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/v0/snippet"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder({"id": id})
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder({"id": id}),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(GetSnippetOut, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncSnippetClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self,
        *,
        page: typing.Optional[int] = None,
        page_size: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetSnippetsResponse:
        """
        Parameters:
            - page: typing.Optional[int]. Page number starting from 1

            - page_size: typing.Optional[int]. Number of items per page

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from athena.client import AsyncAthena

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )
        await client.snippet.get()
        """
        _response = await self._client_wrapper.httpx_client.request(
            method="GET",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/v0/snippets"),
            params=jsonable_encoder(
                remove_none_from_dict(
                    {
                        "page": page,
                        "page_size": page_size,
                        **(
                            request_options.get("additional_query_parameters", {})
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            ),
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(GetSnippetsResponse, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_by_id(self, *, id: str, request_options: typing.Optional[RequestOptions] = None) -> GetSnippetOut:
        """
        Parameters:
            - id: str.

            - request_options: typing.Optional[RequestOptions]. Request-specific configuration.
        ---
        from athena.client import AsyncAthena

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )
        await client.snippet.get_by_id(
            id="snippet_02329f1b-f6ad-4e93-be84-355c33202024b",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            method="POST",
            url=urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/v0/snippet"),
            params=jsonable_encoder(
                request_options.get("additional_query_parameters") if request_options is not None else None
            ),
            json=jsonable_encoder({"id": id})
            if request_options is None or request_options.get("additional_body_parameters") is None
            else {
                **jsonable_encoder({"id": id}),
                **(jsonable_encoder(remove_none_from_dict(request_options.get("additional_body_parameters", {})))),
            },
            headers=jsonable_encoder(
                remove_none_from_dict(
                    {
                        **self._client_wrapper.get_headers(),
                        **(request_options.get("additional_headers", {}) if request_options is not None else {}),
                    }
                )
            ),
            timeout=request_options.get("timeout_in_seconds")
            if request_options is not None and request_options.get("timeout_in_seconds") is not None
            else self._client_wrapper.get_timeout(),
            retries=0,
            max_retries=request_options.get("max_retries") if request_options is not None else 0,  # type: ignore
        )
        if 200 <= _response.status_code < 300:
            return pydantic_v1.parse_obj_as(GetSnippetOut, _response.json())  # type: ignore
        if _response.status_code == 422:
            raise UnprocessableEntityError(
                pydantic_v1.parse_obj_as(HttpValidationError, _response.json())  # type: ignore
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)
